#include "area_transfer.hpp"
#include "godot_cpp/classes/resource_loader.hpp"
#include "godot_cpp/classes/scene_tree.hpp"
#include "godot_cpp/variant/string.hpp"
#include "utils/game_root.hpp"
#include "utils/godot_macros.hpp"

void AreaTransfer::_bind_methods() {
#define CLASSNAME AreaTransfer
    GDPROPERTY_HINTED(next_scene, gd::Variant::STRING, gd::PROPERTY_HINT_FILE, "*.tscn,*.scn");
	GDSIGNAL("progress_changed", gd::PropertyInfo(gd::Variant::INT, "amount_after"));
}

void AreaTransfer::_ready() {
    this->connect("body_entered", callable_mp(this, &self_type::_on_body_entered));
    this->connect("body_exited", callable_mp(this, &self_type::_on_body_exited));
}

void AreaTransfer::_on_body_entered(gd::Node3D *body) {
    if(CharacterUnit *character{gd::Object::cast_to<CharacterUnit>(body)}) {
        this->ready_units.insert(character);
		this->emit_signal("progress_changed", this->ready_units.size());
    }
    this->check_all_ready();
}

void AreaTransfer::_on_body_exited(gd::Node3D *body) {
    CharacterUnit *unit{gd::Object::cast_to<CharacterUnit>(body)};
    if(unit && this->ready_units.has(unit)) {
        this->ready_units.erase(unit);
		this->emit_signal("progress_changed", this->ready_units.size());
    }
}

void AreaTransfer::check_all_ready() {
    gd::TypedArray<gd::Node> units = this->get_tree()->get_nodes_in_group("player_unit");
    for(int i{0}; i < units.size(); ++i) {
        CharacterUnit *unit{gd::Object::cast_to<CharacterUnit>(units[i])};
        if(unit && unit->get_entity_health()->is_conscious() && !this->ready_units.has(unit)) {
            return; // fail
        }
    }
    // no missing units found, switch scenes
    gd::Ref<gd::PackedScene> level{gd::ResourceLoader::get_singleton()->load(this->next_scene)};
    utils::GameRoot3D::get_singleton()->replace_levels(level);
}

void AreaTransfer::set_next_scene(gd::String path) {
    this->next_scene = path;
}

gd::String AreaTransfer::get_next_scene() const {
    return this->next_scene;
}
