#include "objective_flags.hpp"
#include "utils/godot_macros.hpp"

void ObjectiveData::_bind_methods() {
#define CLASSNAME ObjectiveData
	GDSIGNAL("progress_changed");
	GDSIGNAL("is_done");
	GDPROPERTY(name_config, gd::Variant::STRING);
	GDPROPERTY(description, gd::Variant::STRING);
	GDPROPERTY(target_count, gd::Variant::INT);
	GDPROPERTY(completed_count, gd::Variant::INT);
	GDPROPERTY_HINTED(sub_objectives_setting, gd::Variant::ARRAY, gd::PROPERTY_HINT_ARRAY_TYPE, GDRESOURCETYPE("ObjectiveData"));
	GDFUNCTION(get_is_completed);
	GDFUNCTION_ARGS(get_objective_by_name, "name");
}

void ObjectiveData::set_name_config(gd::String name) {
	this->set_name(name);
}

gd::String ObjectiveData::get_name_config() {
	return this->get_name();
}

void ObjectiveData::set_description(gd::String description) {
	this->description = description;
}

gd::String ObjectiveData::get_description() const {
	return this->description;
}

gd::Ref<ObjectiveData> ObjectiveData::get_objective_by_name(gd::StringName name) {
	for(gd::Ref<ObjectiveData> objective : this->get_sub_objectives()) {
		if(objective->get_name() == name) {
			return objective;
		} else {
			gd::Ref<ObjectiveData> sub_objective{objective->get_objective_by_name(name)};
			if(sub_objective.is_valid()) {
				return sub_objective;
			}
		}
	}
	return nullptr;
}

bool ObjectiveData::get_is_completed() const {
	for(gd::Ref<ObjectiveData> sub_objective : this->sub_objectives) {
		if(!sub_objective->get_is_completed()) {
			return false;
		}
	}
	return this->completed_count >= this->target_count;
}

void ObjectiveData::set_target_count(int value) {
	this->target_count = value;
}

int ObjectiveData::get_target_count() const {
	return this->target_count;
}

void ObjectiveData::set_completed_count(int value) {
	this->completed_count = gd::MIN(value, this->target_count);
	this->emit_signal("progress_changed");
}

int ObjectiveData::get_completed_count() const {
	return this->completed_count;
}

void ObjectiveData::set_sub_objectives_setting(gd::Array array) {
	this->sub_objectives.clear();
	for(int i{0}; i < array.size(); ++i) {
		gd::Ref<ObjectiveData> sub_obj_data{array[i]};
		sub_obj_data->connect("progress_changed", callable_mp(this, &self_type::sub_objective_changed));
		this->sub_objectives.push_back(sub_obj_data);
	}
}

gd::Array ObjectiveData::get_sub_objectives_setting() const {
	gd::Array array{};
	for(gd::Ref<ObjectiveData> data : this->sub_objectives) {
		array.push_back(data);
	}
	return array;
}

gd::Vector<gd::Ref<ObjectiveData>> const &ObjectiveData::get_sub_objectives() {
	return this->sub_objectives;
}

void ObjectiveData::sub_objective_changed() {
	this->emit_signal("progress_changed");
}

void ObjectiveData::check_is_done() {
	if(this->get_is_completed()) {
		this->emit_signal("is_done");
	}
}

#undef CLASSNAME

void ObjectiveFlags::_bind_methods() {
#define CLASSNAME ObjectiveFlags
	GDSIGNAL("objectives_changed");
	GDPROPERTY_HINTED(currently_active_objective, gd::Variant::OBJECT, gd::PROPERTY_HINT_RESOURCE_TYPE, "ObjectiveData");
	GDFUNCTION_ARGS(get_objective_by_name, "name");
}


gd::Ref<ObjectiveData> ObjectiveFlags::get_objective_by_name(gd::StringName name) {
	if(this->currently_active_objective->get_name() == name) {
		return this->currently_active_objective;
	} else {
		return this->currently_active_objective->get_objective_by_name(name);
	}
}

void ObjectiveFlags::set_currently_active_objective(gd::Ref<ObjectiveData> value) {
	this->currently_active_objective = value;
}

gd::Ref<ObjectiveData> ObjectiveFlags::get_currently_active_objective() const {
	return this->currently_active_objective;
}
