#ifndef OBJECTIVE_FLAGS_HPP
#define OBJECTIVE_FLAGS_HPP

#include "godot_cpp/templates/vector.hpp"
#include "godot_cpp/variant/array.hpp"
#include <cmath>
#include <godot_cpp/classes/node.hpp>
#include <godot_cpp/classes/resource.hpp>
#include <godot_cpp/templates/hash_set.hpp>

namespace gd = godot;

class ObjectiveData : public gd::Resource {
	GDCLASS(ObjectiveData, gd::Resource);
	static void _bind_methods();
public:
	void set_name_config(gd::String name);
	gd::String get_name_config();
	void set_description(gd::String description);
	gd::String get_description() const;
	gd::Ref<ObjectiveData> get_objective_by_name(gd::StringName name);
	bool get_is_completed() const;
	void set_target_count(int value);
	int get_target_count() const;
	void set_completed_count(int value);
	int get_completed_count() const;
	void set_sub_objectives_setting(gd::Array array);
	gd::Array get_sub_objectives_setting() const;
	gd::Vector<gd::Ref<ObjectiveData>> const &get_sub_objectives();
	void sub_objective_changed();
private:
	void check_is_done();
private:
	int target_count{1};
	int completed_count{0};
	gd::Vector<gd::Ref<ObjectiveData>> sub_objectives{};
	gd::String description{"Undescribed objective"};
};

class ObjectiveFlags : public gd::Node {
	GDCLASS(ObjectiveFlags, gd::Node);
	static void _bind_methods();
public:
	gd::Ref<ObjectiveData> get_objective_by_name(gd::StringName name);
	void set_currently_active_objective(gd::Ref<ObjectiveData> value);
	gd::Ref<ObjectiveData> get_currently_active_objective() const;
private:
	gd::Ref<ObjectiveData> currently_active_objective{};
};

#endif // !OBJECTIVE_FLAGS_HPP
