#include "objective_ui.hpp"
#include <godot_cpp/classes/scene_tree.hpp>
#include <godot_cpp/templates/hash_set.hpp>
#include <godot_cpp/classes/check_box.hpp>
#include "godot_cpp/classes/packed_scene.hpp"
#include "godot_cpp/core/error_macros.hpp"
#include "godot_cpp/core/print_string.hpp"
#include "godot_cpp/templates/pair.hpp"
#include "godot_cpp/variant/variant.hpp"
#include "objective_flags.hpp"
#include "utils/game_root.hpp"
#include "utils/godot_macros.hpp"

void ObjectiveUI::_bind_methods() {
#define CLASSNAME ObjectiveUI
	GDPROPERTY_HINTED(objective_element_scene, gd::Variant::OBJECT, gd::PROPERTY_HINT_RESOURCE_TYPE, "PackedScene");
}

void ObjectiveUI::_ready() {
	flags = utils::GameRoot3D::get_singleton()->get_levels().begin()->value->get_node<ObjectiveFlags>("%ObjectiveFlags");
	flags->connect("objectives_changed", callable_mp(this, &self_type::objectives_changed));
	this->build_ui_from(flags->get_currently_active_objective());
	godot::print_line("ObjectiveUI::_ready");
}

void ObjectiveUI::build_ui_from(gd::Ref<ObjectiveData> data) {
	this->build_ui_element_for(data);
	for(gd::Ref<ObjectiveData> sub_objective : data->get_sub_objectives()) {
		this->build_ui_from(sub_objective);
	}
}

void ObjectiveUI::build_ui_element_for(gd::Ref<ObjectiveData> data) {
	data->connect("progress_changed", callable_mp(this, &self_type::objective_progress).bind(data));
	gd::CheckBox *instance{gd::Object::cast_to<gd::CheckBox>(this->objective_element_scene->instantiate())};
	ERR_FAIL_COND_EDMSG(instance == nullptr, "Failed to instantiate checkbox for objective");
	this->add_child(instance);
	this->display_elements.insert(data, instance);
	this->objective_progress(data);
}

void ObjectiveUI::clear_ui() {
	for(gd::KeyValue<gd::Ref<ObjectiveData>, gd::CheckBox*> kvp : this->display_elements) {
		if(kvp.value != nullptr) {
			kvp.value->queue_free();
		}
	}
	this->display_elements.clear();
}

void ObjectiveUI::objective_progress(gd::Ref<ObjectiveData> data) {
	gd::print_line("ObjectiveUI::objective_progress");
	gd::CheckBox *box{this->display_elements.get(data)};
	box->set_pressed(data->get_is_completed());
	box->set_text(data->get_description() + (data->get_target_count() > 1
		? gd::vformat("(%d/%d)", data->get_completed_count(), data->get_target_count())
		: ""
	));
}

void ObjectiveUI::objectives_changed() {
	this->clear_ui();
	this->build_ui_from(flags->get_currently_active_objective());
}

void ObjectiveUI::set_objective_element_scene(gd::Ref<gd::PackedScene> scene) {
	this->objective_element_scene = scene;
}

gd::Ref<gd::PackedScene> ObjectiveUI::get_objective_element_scene() {
	return this->objective_element_scene;
}
